@@ -1,9 +1,6 @@
 {-# LANGUAGE OverloadedStrings #-}
 module Main where
 
-import Data.Default
-import qualified Graphics.Vty as V
-
 import Brick.Main (App(..), neverShowCursor, resizeOrQuit, defaultMain)
 import Brick.Types
   ( Widget
@@ -23,6 +20,8 @@ import Brick.Widgets.Core
   )
 import Brick.Widgets.Border as B
 import Brick.Widgets.Center as C
+import Brick.AttrMap (attrMap)
+import qualified Graphics.Vty as V
 
 ui :: Widget ()
 ui =
@@ -46,14 +45,13 @@ ui =
          , padAll 2 $ str "Padded by 2 on all sides"
          ]
 
-app :: App () V.Event ()
+app :: App () e ()
 app =
     App { appDraw = const [ui]
         , appHandleEvent = resizeOrQuit
         , appStartEvent = return
-        , appAttrMap = const def
+        , appAttrMap = const $ attrMap V.defAttr []
         , appChooseCursor = neverShowCursor
-        , appLiftVtyEvent = id
         }
 
 main :: IO ()
