@@ -88,6 +88,10 @@ import Data.Monoid (Monoid(..))
 
 import Lens.Micro (_1, _2, to, (^.), (&), (.~), Lens')
 import Lens.Micro.Type (Getting)
+import Control.Monad.Catch (MonadThrow, MonadCatch, MonadMask)
+#if !MIN_VERSION_base(4,13,0)
+import Control.Monad.Fail (MonadFail)
+#endif
 import Control.Monad.Trans.State.Lazy
 import Control.Monad.Trans.Reader
 import Graphics.Vty (Attr)
@@ -128,17 +132,22 @@ handleEventLensed v target handleEvent ev = do
 newtype EventM n a =
     EventM { runEventM :: ReaderT (EventRO n) (StateT (EventState n) IO) a
            }
-           deriving (Functor, Applicative, Monad, MonadIO)
+           deriving ( Functor, Applicative, Monad, MonadIO
+                    , MonadThrow, MonadCatch, MonadMask, MonadFail
+                    )
 
--- | Widget growth policies. These policies communicate to layout
--- algorithms how a widget uses space when being rendered. These
--- policies influence rendering order and space allocation in the box
--- layout algorithm.
+-- | Widget size policies. These policies communicate how a widget uses
+-- space when being rendered. These policies influence rendering order
+-- and space allocation in the box layout algorithm for 'hBox' and
+-- 'vBox'.
 data Size = Fixed
-          -- ^ Fixed widgets take up the same amount of space no matter
-          -- how much they are given (non-greedy).
+          -- ^ Widgets advertising this size policy should take up the
+          -- same amount of space no matter how much they are given,
+          -- i.e. their size depends on their contents alone rather than
+          -- on the size of the rendering area.
           | Greedy
-          -- ^ Greedy widgets take up all the space they are given.
+          -- ^ Widgets advertising this size policy must take up all the
+          -- space they are given.
           deriving (Show, Eq, Ord)
 
 -- | The type of widgets.
