@@ -91,9 +91,11 @@ module Brick.Widgets.Core
   )
 where
 
-#if MIN_VERSION_base(4,8,0)
+#if !(MIN_VERSION_base(4,11,0))
 import Data.Monoid ((<>))
-#else
+#endif
+
+#if !MIN_VERSION_base(4,8,0)
 import Control.Applicative
 import Data.Monoid ((<>), mempty)
 #endif
@@ -243,12 +245,16 @@ takeColumns numCols (c:cs) =
 -- | Make a widget from a string, but wrap the words in the input's
 -- lines at the available width using the default wrapping settings. The
 -- input string should not contain escapes.
+--
+-- Unlike 'str', this is greedy horizontally.
 strWrap :: String -> Widget n
 strWrap = strWrapWith defaultWrapSettings
 
 -- | Make a widget from a string, but wrap the words in the input's
 -- lines at the available width using the specified wrapping settings.
 -- The input string should not contain escapes.
+--
+-- Unlike 'str', this is greedy horizontally.
 strWrapWith :: WrapSettings -> String -> Widget n
 strWrapWith settings t = txtWrapWith settings $ T.pack t
 
@@ -258,12 +264,16 @@ safeTextWidth = V.safeWcswidth . T.unpack
 -- | Make a widget from text, but wrap the words in the input's lines at
 -- the available width using the default wrapping settings. The input
 -- text should not contain escapes.
+--
+-- Unlike 'txt', this is greedy horizontally.
 txtWrap :: T.Text -> Widget n
 txtWrap = txtWrapWith defaultWrapSettings
 
 -- | Make a widget from text, but wrap the words in the input's lines at
 -- the available width using the specified wrapping settings. The input
 -- text should not contain escapes.
+--
+-- Unlike 'txt', this is greedy horizontally.
 txtWrapWith :: WrapSettings -> T.Text -> Widget n
 txtWrapWith settings s =
     Widget Greedy Fixed $ do
@@ -273,13 +283,13 @@ txtWrapWith settings s =
                      | otherwise = l
       case force theLines of
           [] -> return emptyResult
-          [one] -> return $ emptyResult & imageL .~ (V.text' (c^.attrL) one)
           multiple ->
               let maxLength = maximum $ safeTextWidth <$> multiple
+                  padding = V.charFill (c^.attrL) ' ' (c^.availWidthL - maxLength) (length lineImgs)
                   lineImgs = lineImg <$> multiple
                   lineImg lStr = V.text' (c^.attrL)
                                    (lStr <> T.replicate (maxLength - safeTextWidth lStr) " ")
-              in return $ emptyResult & imageL .~ (V.vertCat lineImgs)
+              in return $ emptyResult & imageL .~ (V.horizCat [V.vertCat lineImgs, padding])
 
 -- | Build a widget from a 'String'. Breaks newlines up and space-pads
 -- short lines out to the length of the longest line.
@@ -651,7 +661,7 @@ catDynBorder
 catDynBorder towardsA towardsB a b
     -- Currently, we check if the 'BorderStyle's are exactly the same. In the
     -- future, it might be nice to relax this restriction. For example, if a
-    -- horizontal border is being rewritten to accomodate a neighboring
+    -- horizontal border is being rewritten to accommodate a neighboring
     -- vertical border, all we care about is that the two 'bsVertical's line up
     -- sanely. After all, if the horizontal border's 'bsVertical' is the same
     -- as the vertical one's, and the horizontal border's 'BorderStyle' is
