@@ -53,6 +53,7 @@ module Brick.Widgets.FileBrowser
   , FileInfo(..)
   , FileStatus(..)
   , FileType(..)
+
   -- * Making a new file browser
   , newFileBrowser
   , selectNonDirectories
@@ -102,6 +103,7 @@ module Brick.Widgets.FileBrowser
   , fileInfoSanitizedFilenameL
   , fileInfoFilePathL
   , fileInfoFileStatusL
+  , fileInfoLinkTargetTypeL
   , fileStatusSizeL
   , fileStatusFileTypeL
 
@@ -197,6 +199,9 @@ data FileInfo =
              -- ^ The file status if it could be obtained, or the
              -- exception that was caught when attempting to read the
              -- file's status.
+             , fileInfoLinkTargetType :: Maybe FileType
+             -- ^ If this entry is a symlink, this indicates the type of
+             -- file the symlink points to, if it could be obtained.
              }
              deriving (Show, Eq)
 
@@ -268,6 +273,10 @@ selectNonDirectories :: FileInfo -> Bool
 selectNonDirectories i =
     case fileInfoFileType i of
         Just Directory -> False
+        Just SymbolicLink ->
+            case fileInfoLinkTargetType i of
+                Just Directory -> False
+                _ -> True
         _ -> True
 
 -- | A file entry selector that permits selection of directories
@@ -277,6 +286,10 @@ selectDirectories :: FileInfo -> Bool
 selectDirectories i =
     case fileInfoFileType i of
         Just Directory -> True
+        Just SymbolicLink ->
+            case fileInfoLinkTargetType i of
+                Just Directory -> True
+                _ -> False
         _ -> False
 
 -- | Set the filtering function used to determine which entries in
@@ -335,22 +348,49 @@ getFileInfo :: String
             -- ^ The actual full path to the file or directory to
             -- inspect.
             -> IO FileInfo
-getFileInfo name fullPath = do
-    filePath <- D.makeAbsolute fullPath
-    statusResult <- E.try $ U.getSymbolicLinkStatus filePath
+getFileInfo name = go []
+    where
+        go history fullPath = do
+            filePath <- D.makeAbsolute fullPath
+            statusResult <- E.try $ U.getSymbolicLinkStatus filePath
 
-    let stat = do
-            status <- statusResult
-            let U.COff sz = U.fileSize status
-            return FileStatus { fileStatusFileType = fileTypeFromStatus status
-                              , fileStatusSize = sz
-                              }
+            let stat = do
+                  status <- statusResult
+                  let U.COff sz = U.fileSize status
+                  return FileStatus { fileStatusFileType = fileTypeFromStatus status
+                                    , fileStatusSize = sz
+                                    }
 
-    return FileInfo { fileInfoFilename = name
-                    , fileInfoFilePath = filePath
-                    , fileInfoSanitizedFilename = sanitizeFilename name
-                    , fileInfoFileStatus = stat
-                    }
+            targetTy <- case fileStatusFileType <$> stat of
+                Right (Just SymbolicLink) -> do
+                    targetPathResult <- E.try $ U.readSymbolicLink filePath
+                    case targetPathResult of
+                        Left (_::E.SomeException) -> return Nothing
+                        Right targetPath ->
+                            -- Watch out for recursive symlink chains:
+                            -- if history starts repeating, abort the
+                            -- symlink following process.
+                            --
+                            -- Examples:
+                            --   $ ln -s foo foo
+                            --
+                            --   $ ln -s foo bar
+                            --   $ ln -s bar foo
+                            if targetPath `elem` history
+                            then return Nothing
+                            else do
+                                targetInfo <- liftIO $ go (fullPath : history) targetPath
+                                case fileInfoFileStatus targetInfo of
+                                    Right (FileStatus _ targetTy) -> return targetTy
+                                    _ -> return Nothing
+                _ -> return Nothing
+
+            return FileInfo { fileInfoFilename = name
+                            , fileInfoFilePath = filePath
+                            , fileInfoSanitizedFilename = sanitizeFilename name
+                            , fileInfoFileStatus = stat
+                            , fileInfoLinkTargetType = targetTy
+                            }
 
 -- | Get the file type for this file info entry. If the file type could
 -- not be obtained due to an 'IOException', return 'Nothing'.
@@ -559,6 +599,31 @@ handleFileBrowserEventNormal e b =
 handleFileBrowserEventCommon :: (Ord n) => Vty.Event -> FileBrowser n -> EventM n (FileBrowser n)
 handleFileBrowserEventCommon e b =
     case e of
+        Vty.EvKey (Vty.KChar 'b') [Vty.MCtrl] -> do
+            let old = b ^. fileBrowserEntriesL
+            new <- listMovePageUp old
+            return $ b & fileBrowserEntriesL .~ new
+        Vty.EvKey (Vty.KChar 'f') [Vty.MCtrl] -> do
+            let old = b ^. fileBrowserEntriesL
+            new <- listMovePageDown old
+            return $ b & fileBrowserEntriesL .~ new
+        Vty.EvKey (Vty.KChar 'd') [Vty.MCtrl] -> do
+            let old = b ^. fileBrowserEntriesL
+            new <- listMoveByPages (0.5::Double) old
+            return $ b & fileBrowserEntriesL .~ new
+        Vty.EvKey (Vty.KChar 'u') [Vty.MCtrl] -> do
+            let old = b ^. fileBrowserEntriesL
+            new <- listMoveByPages (-0.5::Double) old
+            return $ b & fileBrowserEntriesL .~ new
+        Vty.EvKey (Vty.KChar 'g') [] ->
+            return $ b & fileBrowserEntriesL %~ listMoveTo 0
+        Vty.EvKey (Vty.KChar 'G') [] -> do
+            let sz = length (listElements $ b^.fileBrowserEntriesL)
+            return $ b & fileBrowserEntriesL %~ listMoveTo (sz - 1)
+        Vty.EvKey (Vty.KChar 'j') [] ->
+            return $ b & fileBrowserEntriesL %~ listMoveBy 1
+        Vty.EvKey (Vty.KChar 'k') [] ->
+            return $ b & fileBrowserEntriesL %~ listMoveBy (-1)
         Vty.EvKey (Vty.KChar 'n') [Vty.MCtrl] ->
             return $ b & fileBrowserEntriesL %~ listMoveBy 1
         Vty.EvKey (Vty.KChar 'p') [Vty.MCtrl] ->
@@ -574,8 +639,16 @@ maybeSelectCurrentEntry b =
             if fileBrowserSelectable b entry
             then return $ b & fileBrowserSelectedFilesL %~ Set.insert (fileInfoFilename entry)
             else case fileInfoFileType entry of
-                Just Directory -> liftIO $ setWorkingDirectory (fileInfoFilePath entry) b
-                _ -> return b
+                Just Directory ->
+                    liftIO $ setWorkingDirectory (fileInfoFilePath entry) b
+                Just SymbolicLink ->
+                    case fileInfoLinkTargetType entry of
+                        Just Directory -> do
+                            liftIO $ setWorkingDirectory (fileInfoFilePath entry) b
+                        _ ->
+                            return b
+                _ ->
+                    return b
 
 selectCurrentEntry :: FileBrowser n -> EventM n (FileBrowser n)
 selectCurrentEntry b =
@@ -730,6 +803,16 @@ fileTypeMatch tys i = maybe False (`elem` tys) $ fileInfoFileType i
 -- regular file with the specified extension. For example, an extension
 -- argument of @"xml"@ would match regular files @test.xml@ and
 -- @TEST.XML@ and it will match directories regardless of name.
+--
+-- This matcher also matches symlinks if and only if their targets are
+-- directories. This is intended to make it possible to use this matcher
+-- to find files with certain extensions, but also support directory
+-- traversal via symlinks.
 fileExtensionMatch :: String -> FileInfo -> Bool
-fileExtensionMatch ext i =
-    ('.' : (toLower <$> ext)) `isSuffixOf` (toLower <$> fileInfoFilename i)
+fileExtensionMatch ext i = case fileInfoFileType i of
+    Just Directory -> True
+    Just RegularFile -> ('.' : (toLower <$> ext)) `isSuffixOf` (toLower <$> fileInfoFilename i)
+    Just SymbolicLink -> case fileInfoLinkTargetType i of
+        Just Directory -> True
+        _ -> False
+    _ -> False
