@@ -461,9 +461,9 @@ listMoveUp :: (Foldable t, Splittable t)
 listMoveUp = listMoveBy (-1)
 
 -- | Move the list selected index up by one page.
-listMovePageUp
-  :: (Foldable t, Splittable t, Ord n)
-  => GenericList n t e -> EventM n (GenericList n t e)
+listMovePageUp :: (Foldable t, Splittable t, Ord n)
+               => GenericList n t e
+               -> EventM n (GenericList n t e)
 listMovePageUp = listMoveByPages (-1::Double)
 
 -- | Move the list selected index down by one. (Moves the cursor down,
@@ -561,7 +561,7 @@ splitClamp l i =
         clamp 0 (if null t then length l - 1 else i) i
 
 -- | Set the selected index for a list to the index of the first
--- occurence of the specified element if it is in the list, or leave
+-- occurrence of the specified element if it is in the list, or leave
 -- the list unmodified otherwise.
 --
 -- /O(n)/.  Only evaluates as much of the container as needed.
