@@ -12,6 +12,7 @@ module Data.Text.Markup
   , fromList
   , fromText
   , toText
+  , isEmpty
   , (@@)
   )
 where
@@ -52,6 +53,10 @@ fromText = (@@ mempty)
 toText :: (Eq a) => Markup a -> T.Text
 toText = T.concat . (fst <$>) . concat . markupToList
 
+-- | Test whether the markup is empty.
+isEmpty :: Markup a -> Bool
+isEmpty (Markup ls) = null ls
+
 -- | Set the metadata for a range of character positions in a piece of
 -- markup. This is useful for, e.g., syntax highlighting.
 markupSet :: (Eq a) => (Int, Int) -> a -> Markup a -> Markup a
